﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7
@EndOfDesignText@
' Using javaobject
' You declare a JavaObject object which then "wraps" any other object and provide three methods:
' SetField, GetField and RunMethod.
' Note that you do not need to specify the parameters types. However the types should be the exact types. 
' The call will fail if you pass a double instead of an int.
' One exception Is that you can pass a string instead of Enum.
' InitializeStatic - Use this method when you want to call a static method or access a static field.
' InitializeNewInstance - Creates a new instance of the given class with the given parameters (or Null)
' RunMethodJO is similar to RunMethod. It returns a JavaObject instead of Object.
' GetFieldJO also returns JavaObject.
Sub Class_Globals
	Private fx As JFX
	Private audioTaggerMP3FileIO As JavaObject
	Private joTagV1, joTagV2 As JavaObject
	Private joFieldKeys As JavaObject
	Private mSourceFolder, mFileName As String
	Private mapTags As Map
	' UDT structure: Tag FiedKey as text; joTagKey is Tag FieldKey as javaobject; ID3V1 tag value retrieved from file; 
	' ID3V2 tag value retrieved from file; PGValue is the value in the form text boxes we want to write to the music file tags.
	Type udtTagRecord ( _
		TagKey As String, _
		joTagKey As JavaObject, _
		ID3V1TagValue As String, _
		ID3V2TagValue As String, _
		PGValue As String _
		)
		
	Public audioMP3File As JavaObject
	' not using these in this test:
	Dim joTag2, joTag2Tag As JavaObject
	Dim joTag2Field, joTag2TagField As JavaObject
	Dim joTag2TextField, joTag2TagTextField As JavaObject
		
End Sub
'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(Folder As String, FileName As String)
	Log("Initializing clIDTag")
	mSourceFolder = Folder
	mFileName = FileName
	' get an instance of audioMP3FileIO so we can read the file
	audioTaggerMP3FileIO.InitializeNewInstance("org.jaudiotagger.audio.AudioFileIO",Null)
	' Set up the FieldKeys
	joFieldKeys.InitializeStatic("org.jaudiotagger.tag.FieldKey")
	' No error - but these are not associated with any specific music file - so not using them
	joTag2.InitializeStatic("org.jaudiotagger.tag.Tag")
	joTag2Field.InitializeStatic("org.jaudiotagger.tag.TagField")
	joTag2TextField.InitializeStatic("org.jaudiotagger.tag.TagTextField")
	' Set up a map of Tags
	mapTags.Initialize
	Setup_mapTags
End Sub

Sub Setup_mapTags
	Try
		Set_MapTag("ALBUM", Main.txfAlbum.Text)
		Set_MapTag("ARTIST", Main.txfArtist.text)
		Set_MapTag("TITLE", Main.txfTrackTitle.Text)
	Catch
		Log($"Exception Message: ${LastException.Message}"$)
	End Try
End Sub

Sub Set_MapTag (sField As String, PGValue As String)
	Try
		Dim TagRecord As udtTagRecord
		TagRecord.Initialize
		TagRecord.ID3V1TagValue = ""
		TagRecord.ID3V2TagValue = ""
		TagRecord.PGValue = PGValue
		TagRecord.joTagKey = joFieldKeys.GetField(sField)
		' Do this one last in Case it errors
		TagRecord.TagKey = sField
	Catch
		Log($"Exception Message: ${LastException.Message}"$)
		TagRecord.TagKey = sField
	End Try
	mapTags.Put(sField, TagRecord)
End Sub
Sub Do_Tags
	Setup_mapTags
	' Check that the file (Folder, Name) exists:
	If File.Exists(mSourceFolder, mFileName) Then
		Dim MP3Fi As JavaObject
		Dim audioMP3Header As JavaObject
		MP3Fi.InitializeNewInstance("java.io.File",Array(File.Combine(mSourceFolder, mFileName)))
		audioMP3File = audioTaggerMP3FileIO.RunMethod("read", Array(MP3Fi))
		' read the headers from audioFile and put into audioHeaders
		audioMP3Header = audioMP3File.RunMethod("getAudioHeader",Null)
		Try
			' Set up the ID3V1 tag javaobject
			joTagV1 = audioMP3File.RunMethod("getID3v1Tag", Null)
			For Each K As String In mapTags.Keys
				Get_TagToMapTags(joTagV1, 1, K)
				Write_MapTagsToMusicFile(joTagV1, 1, K)
			
			Next
			' Set up the ID3V2 tag javaobject converted to ID3TagV2.4
			joTagV2 = audioMP3File.RunMethod("getID3v2TagAsv24", Null)
			For Each K As String In mapTags.Keys
				Get_TagToMapTags(joTagV2, 2, K)
				Write_MapTagsToMusicFile(joTagV2, 2, K)
			Next
		Catch
			Log(LastException.Message)
		End Try
		' Commit the result to the music file - once we get the previous methods to work
		' audioMP3File.RunMethod("commit", Null)
		' or possibly
		' audioTaggerMP3FileIO.RunMethod("write", Array(audioMP3File))
	End If
End Sub

Sub Get_TagToMapTags(joTag As JavaObject, TagType As Int, Key As String)
	Dim TagRecord As udtTagRecord
	Try
		' Get the Tag record for this key
		TagRecord = mapTags.Get(Key)
		Select TagType
			Case 1
				' Get the value from the ID3V1Tag and put it in the TagRecord
				' These approaches all work:
				' Tagrecord.ID3V1TagValue = joTag.RunMethod("getFirst", Array(Key))
				TagRecord.ID3V1TagValue = joTag.RunMethod("getFirst", Array(TagRecord.joTagKey))
				' Tagrecord.ID3V1TagValue = joTag.RunMethod("getFirst", Array(Tagrecord.TagKey))
				' TagRecord.ID3V1TagValue = joTag.RunMethod("getFirstField", Array(TagRecord.joTagKey))

			Case 2
				' Get the value from the ID3V2Tag and put it in the record
				' These approaches all work:
				' Tagrecord.ID3V2TagValue = joTag.RunMethod("getFirst", Array(Key))
				TagRecord.ID3V2TagValue = joTag.RunMethod("getFirst", Array(TagRecord.joTagKey))
				' Tagrecord.ID3V2TagValue = joTag.RunMethod("getFirst", Array(Tagrecord.TagKey))
				' TagRecord.ID3V2TagValue = joTag.RunMethod("getFirstField", Array(TagRecord.joTagKey))

		End Select
		' Don't need to put the record back in the map, it's indirectly referenced
		' mapTags.Put(Key, TagRecord)
	Catch
		If LastException.Message.Contains("No enum constant") Then
			Log($"Enum FieldKey ${LastException.Message.SubString(LastException.Message.LastIndexOf("FieldKey")+9)} does not exist."$)
			Log($"Skipping it."$)
		Else
			Log($"Sub Set_TagTpMapTags: "TagType: ${TagType}, Key: ${Key} ${LastException.Message}"$)
		End If
	End Try
End Sub

Public Sub Write_MapTagsToMusicFile(joTag As JavaObject, TagType As Int, Key As String)
	Dim TagRecord As udtTagRecord
	Try
		' Get the Tag record for this key
		TagRecord = mapTags.Get(Key)
		Select TagType
			Case 1
				' Should be: void setField(FieldKey genericKey, java.lang.String value)
				' This is what I believe should work, but produces Method not matched:
				joTag.RunMethod("setField", Array(TagRecord.joTagKey, TagRecord.PGValue))
				' Or this, but it produces Method not matched:
				' joTag.RunMethod("setField", Array(TagRecord.TagKey, TagRecord.PGValue))
						
				' Or this, but it produces FIELD not found:
				' joTag.SetField(TagRecord.TagKey, TagRecord.PGValue)
			
				' Commit the result to the music file - once we get the previous method to work
				' audioMP3File.RunMethod("commit", Null)

			Case 2
				' Should be: void setField(FieldKey genericKey, java.lang.String value)
				' This is what I believe should work, but produces Method not matched:
				joTag.RunMethod("setField", Array(TagRecord.joTagKey, TagRecord.PGValue))
				' Or this, but it produces Method not matched:
				' joTag.RunMethod("setField", Array(TagRecord.TagKey, TagRecord.PGValue))
						
				' Or this, but it produces FIELD not found:
				' joTag.SetField(TagRecord.TagKey, TagRecord.PGValue)
				
		End Select
	Catch
		If LastException.Message.Contains("No enum constant") Then
			Log($"Enum FieldKey ${LastException.Message.SubString(LastException.Message.LastIndexOf("FieldKey")+9)} does not exist."$)
			Log($"Skipping it."$)
		Else
			Log($"Sub Write_MapTagsToMusicFile: TagType: ${TagType}, Key: ${Key} ${LastException.Message}"$)
		End If
	End Try
End Sub
	
Sub Debug_mapTags
	For Each k As String In mapTags.Keys
		Dim LocalTag As udtTagRecord
		LocalTag = mapTags.Get(k)
		Log($"Key: ${LocalTag.TagKey}, ID3V1TagValue: ${LocalTag.ID3V1TagValue}, ID3V2TagValue: ${LocalTag.ID3V2TagValue} PG Value: ${LocalTag.PGValue}"$)
	Next
End Sub
